/*
 Copyright 2005-2008 Foxmarks Inc.

 foxmarks-dialog.js: Implements the main foxmarks options dialog.

 */

// TO DO:
// * Use synchronize method for preferences

function SetupDialog() {
    var d = document;

    // Set styles for Mac OS X
    if (navigator.platform.toLowerCase().indexOf('mac') > -1 ||
        navigator.platform.toLowerCase().indexOf('linux') > -1) {
        d.getElementById("foxmarks-settings").className = "macosx";
    }

    // Settings that are too complex to handle via prefwindow
    d.getElementById("password").value = gSettings.passwordNoPrompt;
    d.getElementById("synconshutdown").checked = gSettings.syncOnShutdown;
    d.getElementById("askfirst").checked = gSettings.syncOnShutdownAsk;

    // Set server ping
    if (!gSettings.useOwnServer) {
        var attrs = [];
        attrs.push("app="       + "jezebel");
        attrs.push("mid="       + gSettings.machineId);
        attrs.push("page="      + "settings");
        attrs.push("username="  + gSettings.username);
        attrs.push("no_cache="  + Date.now().toString(36));
        var query = attrs.join("&");

        d.getElementById("ping").src = 
            "http://tr.foxmarks.com/tracking/impressions.gif?" + query; 
    }

    // Info that is read-only
    LastSyncDateChanged();
    d.getElementById("version").value = "Foxmarks Bookmark Synchronizer v"
        + FoxmarksVersion();
    try {
        window.sizeToContent();
    } catch(e) {}

    // Make sure we enable-disable as appropriate...
    SyncOnShutdownChanged();
    UseOwnServerChanged();
    ProfileSettingChanged();
}

function StoreSettingsFromDialog() {
    var d = document;

    // Handle complex settings not handled by prefwindow
    gSettings.password = d.getElementById("password").value;
}

function doOK() {
    StoreSettingsFromDialog();
    return true;
}

function doHelp() {
    FoxmarksOpenInNewWindow("http://wiki.foxmarks.com/wiki/Foxmarks:_Help");
}

function FoxmarksForgot() {
    FoxmarksOpenInNewWindow("http://" + gSettings.acctMgrHost + 
            "/login/forgot_password");
}

function UploadNow() {
    var retval = 0;
    if (confirm(Bundle().GetStringFromName("msg.overwriteremote"))) {
        StoreSettingsFromDialog();
        retval = PerformAction("upload", null);
        LastSyncDateChanged();
        LastSyncDateChanged();
    }
    return retval;
}

function DownloadNow(silent) {
    var retval = 0;
    if (silent || confirm(Bundle().GetStringFromName("msg.overwritelocal"))) {
        StoreSettingsFromDialog();
        retval = PerformAction("download", null);
        LastSyncDateChanged();
        LastSyncDateChanged();
    }
    return retval;
}

function SynchronizeNow() {
    var retval = 0;
    StoreSettingsFromDialog();
    var retval = PerformAction("synch", null);
    LastSyncDateChanged();
    return retval;
}

function SynchronizeForever() {
    while (!SynchronizeNow()) { };
}

function RunSetupWizard() {
    StoreSettingsFromDialog();
    FoxmarksOpenWizard(true);
    window.close();
}

function DisplayLogFile() {
    var ios = Cc["@mozilla.org/network/io-service;1"].
        getService(Ci.nsIIOService);
    var file = Cc['@mozilla.org/file/directory_service;1']
        .getService(Ci.nsIProperties) .get('ProfD', Ci.nsIFile);

    file.append("foxmarks.log");
    uri = ios.newFileURI(file);
    FoxmarksOpenInNewWindow(uri.spec);
}

function SyncOnShutdownChanged() {
    document.getElementById("askfirst").disabled =
        !document.getElementById("synconshutdown").checked;
}


function LastSyncDateChanged() {
    document.getElementById("lastSynchDate").value = 
        gSettings.lastSynchDisplayDate;
}

function SyncOnShutdownToPreference() {
    if (document.getElementById("synconshutdown").checked) {
        return 1;
    }
    return 0;
}

function UseOwnServerChanged() {
    var uo = document.getElementById("useown").checked;
    document.getElementById("url").disabled                 = !uo;
    document.getElementById("useown-url-label").disabled    = !uo;
    document.getElementById("encrypt").disabled             = uo;
    document.getElementById("profileChangeButton").disabled       = uo;
}

function FoxmarksChangeProfile() {
    var retval = {};
    window.openDialog("chrome://foxmarks/content/foxmarks-changeprofile.xul",
        "Foxmarks", "modal,centerscreen", retval);

    if (retval.newProfileId && retval.newProfileId != gSettings.viewId) {
        if (confirm(Bundle().GetStringFromName("msg.profilechanged"))) {
            if (SynchronizeNow()) {
                return;
            }
            gSettings.viewId = retval.newProfileId;
            gSettings.viewName = retval.newProfileName;
            DownloadNow(true);
            ProfileSettingChanged();
        }
    }
}

function ProfileSettingChanged() {
    document.getElementById("profileName").value = gSettings.viewName;
}
