/* 
 Copyright 2005-2008 Foxmarks Inc.
 
 foxmarks-progress.js: handles the UI and top-level logic for
 merge, sync, upload, and download operations.
  
 */

var gCancelled;
var gQuick;

var newObserver = {

    observe: function(subject, topic, data) {
        var result = eval(data);
        var element = document && document.getElementById("status");
        LogWrite("Progress: " + data);
        if (element) {
            element.value = result.msg;
            if (result.status != 1) { // complete?
                window.arguments[1].status = result.status;
                window.arguments[1].msg = result.msg;
                window.arguments[1].result = result;
                if (gQuick) {
                    setTimeout(window.close, 1000);
                } else {
                    gCancelled = true;
                    if (result.status == 0 || result.status == 2) {
                        setTimeout(window.close, 100);
                   }
                }
            }
        }        
        this.adjustDialogButtons(result.status);
    },

    adjustDialogButtons: function(status) {
        var d = document.documentElement;
        var help = d.getButton("help");
        help.hidden = (status >= 0 && status <= 2);
        var cancel = d.getButton("cancel");
        cancel.hidden = (status != 1);
        var accept = d.getButton("accept");
        accept.hidden = (status == 1);
        try {
            window.sizeToContent();
        } catch(e) {}
    }
};

var gFoxmarksService = null;
var Cc = Components.classes;
var Ci = Components.interfaces;

function onProgressLoad() {
    gCancelled = false;
    gQuick = false;

    // get reference to foxmarks-service to process the request
    gFoxmarksService = Cc["@foxcloud.com/extensions/foxmarks;1"].
        getService(Ci.nsIFoxmarksService);

    var os = Cc["@mozilla.org/observer-service;1"].
        getService(Ci.nsIObserverService);
    os.addObserver(newObserver, "foxmarks-service", false);
    
    document.getElementById("lastsyncrow").hidden =
        (window.arguments[0] != "showStatus");
  
    var okay = false;

    switch (window.arguments[0]) {
        case "upload":
            gQuick = false;
            okay = gFoxmarksService.upload();
            break;
        
        case "download":
            gQuick = false;
            okay = gFoxmarksService.download();
            break;
        
        case "synch":
            gQuick = false;
            okay = gFoxmarksService.synchronize();
            break;

        case "status":
            gQuick = true;
            okay = gFoxmarksService.status();
            break;
            
        case "initialSync":
            var ca = window.arguments[2];
            gQuick = true;
            okay = gFoxmarksService.synchronizeInitial(ca.remoteIsMaster, 
                ca.merge);
            break;

        case "showStatus":
            var statusElement = document.getElementById("status");
            var lastSyncElement = document.getElementById("lastsync");
            var lastSyncRow = document.getElementById("lastsyncrow");
            var statusRow = document.getElementById("statusrow");
            var status = gFoxmarksService.getLastError();

            if (status) {
                statusElement.value = MapError(status);
            } else {
                statusRow.hidden = true;
            }

            lastSyncElement.value = gSettings.lastSynchDisplayDate;
            newObserver.adjustDialogButtons(status);
            okay = true;
            break;
    }

    if (!okay) {
        // Service is busy. Disconnect the observer and simulate a
        // "busy" status message.
        onProgressUnload();
        newObserver.observe(null, null,
            { msg: Bundle().GetStringFromName("msg.busy"), status: 4 }.
            toSource());
    }
}

function onProgressUnload() {
    var os = Cc["@mozilla.org/observer-service;1"].
        getService(Ci.nsIObserverService);
    try {
        os.removeObserver(newObserver, "foxmarks-service");
    } catch (e) {
        LogWrite("Warning: removeObserver failed.");
    }
}

function onProgressCancel() {
    gFoxmarksService.cancel();
  
    // the first time through, just let callbacks from the cancellation
    // requests above shut us down
    // if we somehow got stuck, allow a second press of the cancel
    // button to actually shut the dialog box down.
    
    if (gCancelled) {
        window.arguments[1].status = -1;
        window.arguments[1].msg = Bundle().GetStringFromName("msg.cancelled");
        return true;
    } else {
        gCancelled = true;
        return false;   // let the callbacks close us down
    }
}

function onProgressHelp() {
    var element = document.getElementById("status");
    if (element) {
        var errmsg = element.value.replace(/ /g, "_");
        window.arguments[1].helpurl =
            Bundle().formatStringFromName("url.error", [errmsg], 1);
        setTimeout(window.close, 100);
    }
    return false;
}
