/* 
 Copyright 2005-2008 Foxmarks Inc.
 
 foxmarks-setup.js: implements behavior for the Foxmarks Setup Wizard.
   
 */


var gIsEmpty;
var gHasProfiles;
var pProfileNames;
var gWizardSession = Date.now().toString(36);
var gHelpUrl;

function HandleError(retval) {
    var wizard = document.documentElement;
    wizard.goTo("errorPage");
    document.getElementById("errormsg").value = retval.msg;
    var errmsg = retval.msg.replace(/ /g, "_");
    gHelpUrl = Bundle().formatStringFromName("url.error", [errmsg], 1);
    wizard.canAdvance = true;
}

function OnTransitionPageShow() {
    /*

    We've just gained control back from the web-based setup wizard.
    1) Confirm that we can log in.
    2) Determine whether there are profiles.
    3) Determine whether there are server-side bookmarks.

    */
    var spinner = document.getElementById("spinner");
    var status = document.getElementById("statusLabel");
    var wizard = document.documentElement;
    wizard.canAdvance = false;
    document.getElementById("profileMenuList").value = String(gSettings.viewId);

    FetchAccountStatus(status, spinner, FetchedStatus);

    function FetchedStatus(response) {
        if (response.status != 0) {
            HandleError(response);
            return;
        }

        gIsEmpty = response.isreset == "true" || response.isreset == true;
        wizard.getPageById("selectProfile").next = gIsEmpty ? 
            "execute" : "selectSyncOption"; 

        FetchProfileNames(status, spinner,
            document.getElementById("profileMenuPopup"),
            FetchedProfileNames);
    }

    function FetchedProfileNames(response) {
        if (response.status != 0) {
            HandleError(response);
            return;
        }

        gHasProfiles = (response.count > 0);
        gProfileNames = response.profiles;
        wizard.getPageById("transition").next = 
            (gHasProfiles && !gSettings.haveSynced) ?
            "selectProfile" : wizard.getPageById("selectProfile").next;
        wizard.canAdvance = true;
        wizard.advance();
    }
}

function SetProfileValue() {
    LogWrite("Setting profile value...");
    gSettings.viewId = document.getElementById("profileMenuList").value;
    gSettings.viewName = document.getElementById("profileMenuList").label;
    LogWrite("Profle value is now " + gSettings.viewId);
}

function SyncOptionAdvance() {
    var wizard = document.documentElement;
    wizard.currentPage.next = SetupIsMerging() ? "mergeOption" : "execute";
    return true;
}

function FoxmarksSetupHelp() {
    if (gHelpUrl) {
        FoxmarksOpenInNewWindow(gHelpUrl);
    }
}

function SetupIsMerging() {
    return !gIsEmpty && 
        document.getElementById("localOrRemote").selectedItem.value == "merge";
}

// Skip over merge options if the user hasn't selected merge.
function SetupOptionNext() {
    document.documentElement.getPageById("selectSyncOption").next = 
        SetupIsMerging() ? "mergeOption" : "execute";
    return true;
}

function SetupShowExecutePage() {
    var op;
    var a = document.getElementById("localOrRemote").selectedItem.value;
    var b = document.getElementById("mergeStart").selectedItem.value;

    if (gIsEmpty) {
        op = "msg.upload";
    } else {
        if (a == "local") {
            op = "msg.upload";
        } else if (a == "remote") {
            op = "msg.download";
        } else {
            if (b == "local") {
                op = "msg.mergelocal";
            } else {
                op = "msg.mergeremote";
            }
        }
    }
    document.getElementById("operation").value = Bundle().GetStringFromName(op);
    var warning = document.getElementById("warning");
    if (!gIsEmpty && a != "merge") {
        warning.value = Bundle().GetStringFromName(op + ".warning");
        warning.hidden = false;
    } else {
        warning.hidden = true
    }
    var profileMsg = document.getElementById("profileMsg");
    if (gHasProfiles && gSettings.viewId) {
        profileMsg.value = Bundle().formatStringFromName("msg.profilemsg",
            [gProfileNames[String(gSettings.viewId)]], 1);
        profileMsg.hidden = false;
    } else {
        profileMsg.hidden = true;
    }
}

function SetupPerformSync() {
    var retval = {}
    var args = {};

    var a = document.getElementById("localOrRemote").selectedItem.value;
    var b = document.getElementById("mergeStart").selectedItem.value;

    args.merge = SetupIsMerging();
    args.remoteIsMaster = gIsEmpty ? false :
        (args.merge ? (b == "remote") : (a == "remote"));

    SetupPerformAction("initialSync", retval, args);

    if (!retval.status) {
        return true;
    } else {
        alert(Bundle().formatStringFromName("msg.syncfailed",
                [retval.msg], 1));
    }

    return false;
}

function SetupPerformAction(action, retval, args) {
    var win = window.
            openDialog("chrome://foxmarks/content/foxmarks-progress.xul",
            "_blank", "chrome,dialog,modal,centerscreen", action, retval, args);

    if (retval.helpurl) {
        openDialog("chrome://browser/content/browser.xul", "_blank",
            "chrome,all,dialog=no", retval.helpurl);
        retval.status = -1;
        retval.msg = Bundle().GetStringFromname("msg.cancelled");
    }

    return;
}

function SetupOnWizardFinish() {
    gSettings.majorVersion = 2;
    FoxmarksOpenInNewTab(gSettings.acctMgrHost + "/success");
    return true;
}

function OnPageShow(pageId) {
    var attrs = [];
    attrs.push("app="       + "jezebel");
    attrs.push("mid="       + gSettings.machineId);
    attrs.push("sess="      + gWizardSession);
    attrs.push("page="      + pageId);
    attrs.push("username="  + gSettings.username);
    attrs.push("no_cache="  + Date.now().toString(36));
    attrs.push("manual="    + window.arguments[0]);

    var query = attrs.join("&");
    var img = document.getElementById(pageId + "Ping");
    img.src = "http://tr.foxmarks.com/tracking/impressions.gif?" + query;
}
